// Package
package zeroslackr.installer;

// Imports
import java.io.*;
import java.util.zip.*;
import com.ice.tar.*;

/**
 * Abstract Extractor Class.
 * <p>
 * Methods for extracting archives.
 * <p>
 * Licensed under <A HREF="http://www.gnu.org/licenses/gpl.txt">GNU GPL (Version 3)</A>.
 * <p>
 * Sources: 
 * <A HREF="http://java.sun.com/developer/technicalArticles/Programming/compression/"> SDN "Compressing and Decompressing Data using Java"</A>, 
 * <A HREF="http://www.gjt.org/javadoc/com/ice/tar/package-summary.html">ICE Engineering's tar package API</A>, 
 * <A HREF="http://jigsaw.w3.org/Doc/Programmer/api/org/w3c/tools/crypt/Md5.html">Jigsaw's Md5 class API</A>
 * <p>
 * @author	Keripo
 * @version	Jan 18, 2008
 */

public abstract class Extractor {
	
	/**
	 * Extracts an archive to a specified location. 
	 * Supported file extensions: .gz, .tar.gz, .zip<br>
	 *   ie extract("kernel.gz", "/boot/linux.bin");<br>
	 *   or extract("pz2modules.tar.gz", "/usr/lib");<br>
	 *   or extract("idoom.zip", "/opt");
	 * @param	file	String of file location.
	 * @param	dest	String of destination folder/file.
	 * @return	True if successful, false otherwise.
	 */
	public static boolean extract(String file, String dest) throws IOException {
		String fileLower = file.toLowerCase();
		if (fileLower.endsWith(".tar.gz")) {
			extractTarGz(file, dest);
			return true;
		} else if (fileLower.endsWith(".gz")) {
			extractGz(file, dest);
			return true;
		} else if (fileLower.endsWith(".zip")) {
			extractZip(file, dest);
			return true;
		} else {
			return false;
		}
	} // extract

	/**
	 * Extracts .zip archives.
	 * @see #extract(String file, String dest)
	 */
	private static void extractZip(String file, String dest) throws IOException {
		
		// Create directories
		File destFolder = new File(Tools.convertPath(dest));
		destFolder.mkdirs();
		
		// Create input stream
		ZipInputStream in = new ZipInputStream(new FileInputStream(
				new File(Tools.convertPath(file))));
		
		// Extract
		byte[] buffer = new byte[256]; // Arbitrary length
		ZipEntry e;
		while ((e = in.getNextEntry()) != null) {
			// Extraction file path
			File destFile = new File(
					Tools.convertPath(destFolder.getPath() + "/" + e.getName()));
			// Directory
			if (e.isDirectory()) {
				destFile.mkdirs();
			// File
			} else {
				FileOutputStream out = new FileOutputStream(destFile);
				int len;
		        while ( ( len = in.read(buffer) ) != -1 ) {
		        	out.write(buffer, 0, len);
		        }
				out.close();
			}
		}
		
        // Cleanup
        in.close();
		
	} // extractZip
	
	/**
	 * Extracts .gz archives.
	 * @see #extract(String file, String dest)
	 */
	private static void extractGz (String file, String dest) throws IOException {
		
		// Create file/folders
		File d = new File(Tools.convertPath(dest.substring(0, dest.lastIndexOf('/'))));
		d.mkdirs();
		File f = new File(Tools.convertPath(dest));
		f.createNewFile();
		
		// Create streams
		GZIPInputStream in = new GZIPInputStream(new FileInputStream(
				new File(Tools.convertPath(file))));
		FileOutputStream out = new FileOutputStream(f);
		
		// Extract
        byte[] buffer = new byte[256]; // Arbitrary length
        int len;
        while ((len = in.read(buffer)) != -1) {
        	out.write(buffer, 0, len);
        }
        
        // Cleanup
        in.close();
        out.close();
		
	} // extractGz
	
	/**
	 * Extracts .tar.gz archives.
	 * @see #extract(String file, String dest)
	 */
	private static void extractTarGz (String file, String dest) throws IOException {
		
		// Create directories
		File destFolder = new File(Tools.convertPath(dest));
		destFolder.mkdirs();
		
		// Create input stream
		TarInputStream in = new TarInputStream(new GZIPInputStream(
				new FileInputStream(new File(Tools.convertPath(file)))));
		
		// Extract
		TarEntry e;
		while ((e = in.getNextEntry()) != null) {
			// Extraction file path
			File destFile = new File(
					Tools.convertPath(destFolder.getPath() + "/" + e.getName()));
			// Directory
			if (e.isDirectory()) {
				destFile.mkdirs();
			// File
			} else {
				FileOutputStream out = new FileOutputStream(destFile);
				in.copyEntryContents(out);
				out.close();
			}
		}
		
		// Cleanup
		in.close();
		
	} // extractTarGz
	
} // Extractor
