// Package
package zeroslackr.installer;

// Imports
import java.util.*;
import java.io.*;
import org.ini4j.*;
import org.ini4j.Ini.Section;

/**
 * List Class.
 * <p>
 * List objects that create Pack objects.
 * <p>
 * Licensed under <A HREF="http://www.gnu.org/licenses/gpl.txt">GNU GPL (Version 3)</A>.
 * <p> 
 * Sources: 
 * <A HREF="http://ini4j.sourceforge.net/api/index.html">ini4j's API</A>,
 * <p>
 * @see Pack
 * @author	Keripo
 * @version	Jan 18, 2008
 */

// List class
public class List {

	// INI file
		Ini ini;
		String file;
	
	// Lists
		// Disable Loader2 for now
		//static ArrayList<Pack> loader = new ArrayList<Pack>();
		ArrayList<Pack> kernel = new ArrayList<Pack>();
		ArrayList<Pack> userland = new ArrayList<Pack>();
		ArrayList<Pack> launcher = new ArrayList<Pack>();
		ArrayList<Pack> packs = new ArrayList<Pack>();
		ArrayList<Pack> extras = new ArrayList<Pack>();
	
	/**
	 * Constructor.
	 * @param	file Path to special INI file.
	 */
	public List(String file)
		throws IOException, InvalidListFormatException {
		
		try {
			
			// Setup
			this.file = file;
			ini = new Ini();
			ini.load(new FileInputStream(new File(Tools.convertPath(file))));
			
			// Special index section
			Section index = ini.get("index");
			if (index == null) {
				throw new InvalidListFormatException(
					"No \"index\" section found in List \"" + file + "\".");
			}
			
			// Populate ArrayLists
			//populate(index, loader, "loader");
			populate(index, kernel, "kernel");
			populate(index, userland, "userland");
			populate(index, launcher, "launcher");
			populate(index, packs, "packs");
			populate(index, extras, "extras");
		
		} catch (InvalidIniFormatException e) {
			throw new InvalidListFormatException(
					"Improper INI file format for List \"" + file + "\".\n" + e.getMessage());
		}
		
	} // constructor
	
	/**
	 * Retrieves list of packs from the index.
	 * @see #List(String iniFile)
	 */
	private void populate(Section index, ArrayList<Pack> list, String type)
		throws InvalidListFormatException {
		String buffer = index.fetch(type);
		if (!(buffer == null) && !buffer.equals("")) {
			for (String item: buffer.split(" ")) {
				list.add(createPack(item, type));
			}
		}
	} // populate
	
	/**
	 * Creates packs from the lists.
	 */
	private Pack createPack(String item, String type)
		throws InvalidListFormatException {
		
		// Get section for the pack
		Section sec = ini.get(item);
		if (sec == null) {
			throw new InvalidListFormatException(
				"Section \"" + item + "\" not found" + " in List \"" + file + "\".");
		}
		
		// Create pack 	
		Pack p = new Pack();
		
		// Add info
			// Ini info
			p.pack			= sec.getName();
			p.list			= file;
			p.type			= type;
			
			// About
			p.name			= sec.fetch("name");
			p.author		= sec.fetch("author");
			p.category		= sec.fetch("category");
			
			// Stats
			//p.status		= sec.fetch("status");
			//p.supported		= sec.fetch("supported").split(" ");
			//p.dependencies	= sec.fetch("dependencies").split(" ");
			
			// Info
			p.version		= sec.fetch("version");
			p.description	= sec.fetch("description");
			//p.icon			= sec.fetch("icon");
			//p.links			= sec.fetch("links").split(" ");
			//p.pictures		= sec.fetch("pictures").split(" ");
			
			// Download
			p.download		= sec.fetch("download");
			p.Md5			= sec.fetch("Md5");
			p.path			= sec.fetch("path");
			//p.mirrors		= sec.fetch("mirrors").split(" ");
		
		// Return
		return p;
		
	} // createPack

} // List class

/**
 * InvalidListFormatException exception.
 * <p>
 * Exception thrown by incorrect List format.
 * <p>
 * Licensed under <A HREF="http://www.gnu.org/licenses/gpl.txt">GNU GPL (Version 3)</A>. 
 * <p>
 * @see List
 * @author	Keripo
 * @version	Jan 14, 2008
 */

class InvalidListFormatException extends Exception {
	
	private static final long serialVersionUID = 1L;
	private String msg;
	
	/**
	 * Construtor with a meaningful error message.
	 * @param	msg	Message containing info about the cause of the error.
	 */
	public InvalidListFormatException(String msg) {
		this.msg = msg;
	} // constructor
	
	/**
	 * @return A meaningful error message.
	 */
	public String getMessage() {
		return msg;
	} // getMessage
	
} // InvalidListFormatException exception
