// Package
package zeroslackr.installer;

// Imports
import java.io.*;
import java.util.Scanner;

/**
 * iPodPatcher Class.
 * <p>
 * Use external ipodpatcher binary to install Loader2. Note that since ipodpatcher 
 * is written in C++, I may be able to port it over to Java later.
 * <p>
 * Licensed under <A HREF="http://www.gnu.org/licenses/gpl.txt">GNU GPL (Version 3)</A>. 
 * <p>
 * Sources: 
 * <A HREF="http://www.rockbox.org/twiki/bin/view/Main/IpodPatcher">Rockbox's iPodPatcher wiki page</A>, 
 * <A HREF="svn://svn.rockbox.org/rockbox/trunk/rbutil/ipodpatcher/">Rockbox's iPodPatcher SVN source code</A>
 * <p>
 * @author	Keripo
 * @version	Jan 18, 2008
 */

public class iPodPatcher {

	// Variables
	private File ipodpatcher;
	
	/**
	 * Constructor.
	 * @param	folder	Path to folder containing ipodpatcher binaries
	 */
	public iPodPatcher(String folder) {
		ipodpatcher = new File(getPath(folder));
	} // constructor
	
	/**
	 * @param	folder	Path to folder containing ipodpatcher binaries.
	 * @return	Path to ipodpatcher binary, null otherwise.
	 */
	private static String getPath(String folder) {
		switch (Tools.getHostOS()) {
			case -1: return null;
			case  0: return null;
			case 11: return Tools.convertPath(folder + "/linux32x86/ipodpatcher");
			case 12: return Tools.convertPath(folder + "/linux64amd64/ipodpatcher");
			case 13: return Tools.convertPath(folder + "/linux-pcc/ipodpatcher");
			case 21: return Tools.convertPath(folder + "/win32/ipodpatcher.exe");
			case 31: return Tools.convertPath(folder + "/macosx/ipodpatcher.dmg");
			default: return null;
		}
	} // getPath
	
	/**
	 * Check if the ipodpatcher binary exists.
	 * @return	True if ipodpatcher binary exists, false otherwise.
	 */
	public boolean exists() {
		return ipodpatcher.exists();
	} // exists
	
	/**
	 * Execute ipodpatcher binary with arguments.
	 * @return	Pointer to newly-created process.
	 */
	public Process exec(String[] args) throws IOException {
		String[] cmd = new String[args.length+1];
		cmd[0] = ipodpatcher.getPath();
		for (int i = 0; i < args.length; i++) {
			cmd[i+1] = args[i];
		}
		return Runtime.getRuntime().exec(cmd);	
	} // exec
	
	/**
	 * Get ipodpatcher's error stream for parsing. Primative but works. 
	 * (Unable to use .exitValue() since errorlevel reporting needs
	 *  to be fixed even in the SVN code; I will probably be submitting
	 *  a patch to the Rockbox developers later). 
	 * @param	p	ipodpatcher process.
	 * @return	String containing the ipodpatcher's entire error stream.
	 */
	private static String getErrOutput(Process p) {
		Scanner in = new Scanner(p.getErrorStream());
		String output = "";
		while (in.hasNextLine()) {
			output += in.nextLine() + "\n";
		}
		return output;
	} // getErrOutput

	/**
	 * Backup iPod's firmware to an image.
	 * @param	path	Path to backup location.
	 */
	public void backup(String path)
		throws IOException, InterruptedException, iPodPatcherFailureException {
		String cmd[] = {"--read-partition", Tools.convertPath(path)};
		String output = getErrOutput(exec(cmd));
		// ipodpatcher code: "fprintf(stderr,"[INFO] Partition extracted to %s.\n",filename);"
		if (output.indexOf("[INFO] Partition extracted to") == -1) {
			throw new iPodPatcherFailureException(output);
		}
	} // backup
	
	/**
	 * Restore iPod's firmware from an image.
	 * @param	path	Path to backup location.
	 */
	public void restore(String path)
		throws IOException, InterruptedException, iPodPatcherFailureException {
		String cmd[] = {"--write-partition", Tools.convertPath(path)};
		String output = getErrOutput(exec(cmd));
		// ipodpatcher code: "fprintf(stderr,"[INFO] %s restored to partition\n",filename);"
		if (output.indexOf("restored to partition") == -1) {
			throw new iPodPatcherFailureException(output);
		}
	} // restore
	
	/**
	 * Install Loader 2 from file.
	 * @param	path	Path to loader.bin location.
	 */
	public void install(String path)
		throws IOException, InterruptedException, iPodPatcherFailureException {
		String cmd[] = {"--add-bootloader-bin", Tools.convertPath(path)};
		String output = getErrOutput(exec(cmd));
		// ipodpatcher code: "fprintf(stderr,"[INFO] Bootloader %s written to device.\n",filename);"
		if (output.indexOf("[INFO] Bootloader ") == -1 && output.indexOf("written to device.") == -1) {
			throw new iPodPatcherFailureException(output);
		}
	} // install
	
} // iPodPatcher class


/**
 * iPodPatcherFailureException exception.
 * <p>
 * Exception thrown by ipodpatcher failure.
 * <p>
 * Licensed under <A HREF="http://www.gnu.org/licenses/gpl.txt">GNU GPL (Version 3)</A>. 
 * <p>
 * @see List
 * @author	Keripo
 * @version	Jan 14, 2008
 */

class iPodPatcherFailureException extends Exception {
	
	private static final long serialVersionUID = 1L;
	private String msg;
	
	/**
	 * Construtor with a meaningful error message.
	 * @param	msg	Message containing info about the cause of the error.
	 */
	public iPodPatcherFailureException(String msg) {
		this.msg = "ipodpatcher output:\n" + msg;
	} // constructor
	
	/**
	 * @return A meaningful error message.
	 */
	public String getMessage() {
		return msg;
	} // getMessage
	
} // iPodPatcherFailureException exception
